from typing import Optional, Union

from azure.core.paging import ItemPaged
from azure.identity import DefaultAzureCredential

from cloud_governance.cloud_resource_orchestration.utils.common_operations import string_equal_ignore_case
from cloud_governance.main.environment_variables import environment_variables


class CommonOperations:

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self._default_creds = DefaultAzureCredential()
        self._subscription_id = self.__environment_variables_dict.get('AZURE_SUBSCRIPTION_ID')

    def _item_paged_iterator(self, item_paged_object: ItemPaged):
        """
        This method iterates the paged object and return the list
        :param item_paged_object:
        :return:
        """
        iterator_list = []
        try:
            page_item = item_paged_object.next()
            while page_item:
                iterator_list.append(page_item)
                page_item = item_paged_object.next()
        except StopIteration:
            pass
        return iterator_list

    def check_tag_name(self, tags: Optional[dict], tag_name: str):
        """
        This method checks tag is present and return its value
        :param tags:
        :param tag_name:
        :return:
        """
        if tags:
            for key, value in tags.items():
                if string_equal_ignore_case(key, tag_name):
                    return value
        return ''



from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.tag_cro_resources import TagCROInstances


class AzureRunCro:

    def __init__(self):
        pass

    def __run_cloud_resources(self):
        """
        This method run the azure resources in specified region or all regions
        :return:
        """
        TagCROInstances().run()

    def __start_cro(self):
        """
        This method start the cro process methods
        1. Send alert to cost over usage users
        2. Tag the new instances
        3. monitor and upload the new instances' data
        4. Monitor the Jira ticket progressing
        :return:
        """
        self.__run_cloud_resources()

    def run(self):
        """
        This method start the Azure CRO operations
        :return:
        """
        self.__start_cro()



from azure.mgmt.compute import ComputeManagementClient

from cloud_governance.common.clouds.azure.compute.common_operations import CommonOperations
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp


class ComputeOperations(CommonOperations):

    def __init__(self):
        super().__init__()
        self.__compute_client = ComputeManagementClient(self._default_creds, subscription_id=self._subscription_id)

    @logger_time_stamp
    def get_instances_by_resource_group(self, resource_group_name: str):
        """
        This method returns all the compute resources by resource group
        :return:
        """
        instances_paged_object = self.__compute_client.virtual_machines.list(resource_group_name=resource_group_name)
        instances_list = self._item_paged_iterator(item_paged_object=instances_paged_object)
        return instances_list

